@@ -9,8 +9,6 @@
 #include "controller.h"
 #include "eventhandler.h"
 #include "models/actionsmodel.h"
-#include "models/messagefiltermodel.h"
-#include "models/sortfilterroomtreemodel.h"
 #include "neochatconfig.h"
 #include "neochatconnection.h"
 #include "neochatroom.h"
@@ -43,6 +41,7 @@ RoomManager::RoomManager(QObject *parent)
     , m_messageFilterModel(new MessageFilterModel(this, m_timelineModel))
     , m_mediaMessageFilterModel(new MediaMessageFilterModel(this, m_messageFilterModel))
     , m_userListModel(new UserListModel(this))
+    , m_widgetModel(new WidgetModel(this))
 {
 #if defined(Q_OS_ANDROID) || defined(Q_OS_IOS) || defined(UBUNTU_TOUCH)
     m_isMobile = true;
@@ -54,11 +53,8 @@ RoomManager::RoomManager(QObject *parent)
     }
 #endif
 
-    m_lastRoomConfig = m_config->group(u"LastOpenRoom"_s);
-    m_lastSpaceConfig = m_config->group(u"LastOpenSpace"_s);
-    m_directChatsConfig = m_config->group(u"DirectChatsActive"_s);
-
     connect(this, &RoomManager::currentRoomChanged, this, [this]() {
+        m_widgetModel->setRoom(m_currentRoom);
         m_userListModel->setRoom(m_currentRoom);
         m_timelineModel->setRoom(m_currentRoom);
         m_sortFilterRoomTreeModel->setCurrentRoom(m_currentRoom);
@@ -201,6 +197,11 @@ void RoomManager::activateUserModel()
     m_userListModel->activate();
 }
 
+WidgetModel *RoomManager::widgetModel() const
+{
+    return m_widgetModel;
+}
+
 void RoomManager::resolveResource(const QString &idOrUri, const QString &action)
 {
     resolveResource(Uri{idOrUri}, action);
@@ -213,34 +214,53 @@ void RoomManager::resolveResource(Uri uri, const QString &action)
         return;
     }
 
-    if (uri.type() == Uri::NonMatrix && action == "qr"_L1) {
-        Q_EMIT externalUrl(uri.toUrl());
-        return;
+    if (action == "qr"_L1) {
+        if (uri.type() == Uri::NonMatrix) {
+            Q_EMIT externalUrl(uri.toUrl());
+            return;
+        }
+        if (uri.type() != Uri::UserId) {
+            uri.setAction(QStringLiteral("join"));
+        } else {
+            uri.setAction(action);
+        }
     }
 
+    // For matrix URIs:
     if (uri.type() != Uri::NonMatrix) {
         if (!m_connection) {
             return;
         }
-        if (!action.isEmpty() && (uri.type() != Uri::UserId || action != "join"_L1)) {
-            uri.setAction(action);
+        // Once a join is confirmed, set the action to "join" so it skips the confirmation check.
+        if (action == "join_confirmed"_L1) {
+            uri.setAction(QStringLiteral("join"));
         }
         // TODO we should allow the user to select a connection.
     }
 
     const auto result = visitResource(m_connection, uri);
+
+    // If we are not already in the room:
     if (result == Quotient::CouldNotResolve) {
-        if ((uri.type() == Uri::RoomAlias || uri.type() == Uri::RoomId) && action != "no_join"_L1) {
+        if ((uri.type() == Uri::RoomAlias || uri.type() == Uri::RoomId) && action == "join"_L1) {
             Q_EMIT askJoinRoom(uri.primaryId());
         }
     }
 }
 
-void RoomManager::maximizeMedia(int index)
+void RoomManager::maximizeMedia(const QString &eventId)
 {
-    if (index < -1 || index > m_mediaMessageFilterModel->rowCount()) {
+    if (eventId.isEmpty()) {
+        qWarning() << "Tried to open media for empty event id";
         return;
     }
+
+    const auto index = m_mediaMessageFilterModel->getRowForEventId(eventId);
+    if (index == -1) {
+        qWarning() << "Tried to open media for unknown event id" << eventId;
+        return;
+    }
+
     Q_EMIT showMaximizedMedia(index);
 }
 
@@ -262,27 +282,28 @@ void RoomManager::viewEventSource(const QString &eventId)
     Q_EMIT showEventSource(eventId);
 }
 
-void RoomManager::viewEventMenu(const QString &eventId, NeoChatRoom *room, NeochatRoomMember *sender, const QString &selectedText, const QString &hoveredLink)
+void RoomManager::viewEventMenu(const QString &eventId, NeoChatRoom *room, const QString &selectedText, const QString &hoveredLink)
 {
-    const auto &event = **room->findInTimeline(eventId);
-
-    if (EventHandler::mediaInfo(room, &event).contains("mimeType"_L1)) {
-        Q_EMIT showFileMenu(eventId,
-                            sender,
-                            MessageComponentType::typeForEvent(event),
-                            EventHandler::plainBody(room, &event),
-                            EventHandler::mediaInfo(room, &event)["mimeType"_L1].toString(),
-                            room->fileTransferInfo(eventId));
+    if (eventId.isEmpty()) {
+        qWarning() << "Tried to open event menu with empty event id";
         return;
     }
 
-    Q_EMIT showMessageMenu(eventId,
-                           sender,
-                           MessageComponentType::typeForEvent(event),
-                           EventHandler::plainBody(room, &event),
-                           EventHandler::richBody(room, &event),
-                           selectedText,
-                           hoveredLink);
+    const auto it = room->findInTimeline(eventId);
+    if (it == room->historyEdge()) {
+        // This is probably a pending event
+        return;
+    }
+    const auto &event = **it;
+    Q_EMIT showDelegateMenu(eventId,
+                            room->qmlSafeMember(event.senderId()),
+                            MessageComponentType::typeForEvent(event),
+                            EventHandler::plainBody(room, &event),
+                            EventHandler::richBody(room, &event),
+                            EventHandler::mediaInfo(room, &event)["mimeType"_L1].toString(),
+                            room->fileTransferInfo(eventId),
+                            selectedText,
+                            hoveredLink);
 }
 
 bool RoomManager::hasOpenRoom() const
@@ -304,7 +325,7 @@ void RoomManager::loadInitialRoom()
     }
 
     if (m_isMobile) {
-        QString lastSpace = m_lastSpaceConfig.readEntry(m_connection->userId(), QString());
+        QString lastSpace = m_lastRoomConfig.readEntry(u"lastSpace"_s, QString());
         // We can't have empty keys in KConfig, so we stored it as "Home"
         if (lastSpace == u"Home"_s) {
             lastSpace.clear();
@@ -331,7 +352,11 @@ void RoomManager::openRoomForActiveConnection()
         setCurrentSpace({}, false);
         return;
     }
-    setCurrentSpace(m_lastSpaceConfig.readEntry(m_connection->userId(), QString()), true);
+    auto lastSpace = m_lastRoomConfig.readEntry(u"lastSpace"_s, QString());
+    if (lastSpace == u"Home"_s) {
+        lastSpace.clear();
+    }
+    setCurrentSpace(lastSpace, true);
 }
 
 UriResolveResult RoomManager::visitUser(User *user, const QString &action)
@@ -358,19 +383,6 @@ void RoomManager::visitRoom(Room *r, const QString &eventId)
     if (m_currentRoom && !m_currentRoom->editCache()->editId().isEmpty()) {
         m_currentRoom->editCache()->setEditId({});
     }
-    if (m_currentRoom && !m_currentRoom->isSpace() && m_chatDocumentHandler) {
-        // We're doing these things here because it is critical that they are switched at the same time
-        if (m_chatDocumentHandler->document()) {
-            m_currentRoom->mainCache()->setSavedText(m_chatDocumentHandler->document()->textDocument()->toPlainText());
-            m_chatDocumentHandler->setRoom(room);
-            if (room) {
-                m_chatDocumentHandler->document()->textDocument()->setPlainText(room->mainCache()->savedText());
-                room->mainCache()->setText(room->mainCache()->savedText());
-            }
-        } else {
-            m_chatDocumentHandler->setRoom(room);
-        }
-    }
 
     if (!room) {
         setCurrentRoom({});
@@ -404,7 +416,7 @@ void RoomManager::joinRoom(Quotient::Connection *account, const QString &roomAli
         job.get(),
         &Quotient::BaseJob::finished,
         this,
-        [this, account](Quotient::BaseJob *finish) {
+        [this, account, roomAliasOrId](Quotient::BaseJob *finish) {
             if (finish->status() == Quotient::BaseJob::Success) {
                 connect(
                     account,
@@ -415,7 +427,7 @@ void RoomManager::joinRoom(Quotient::Connection *account, const QString &roomAli
                     },
                     Qt::SingleShotConnection);
             } else {
-                Q_EMIT showMessage(MessageType::Warning, i18n("Failed to join room<br />%1", finish->errorString()));
+                Q_EMIT showMessage(MessageType::Warning, i18n("Failed to join %1:<br />%2", roomAliasOrId, finish->errorString()));
             }
         },
         Qt::SingleShotConnection);
@@ -470,18 +482,6 @@ bool RoomManager::visitNonMatrix(const QUrl &url)
     return true;
 }
 
-ChatDocumentHandler *RoomManager::chatDocumentHandler() const
-{
-    return m_chatDocumentHandler;
-}
-
-void RoomManager::setChatDocumentHandler(ChatDocumentHandler *handler)
-{
-    m_chatDocumentHandler = handler;
-    m_chatDocumentHandler->setRoom(m_currentRoom);
-    Q_EMIT chatDocumentHandlerChanged();
-}
-
 void RoomManager::setConnection(NeoChatConnection *connection)
 {
     if (m_connection == connection) {
@@ -495,6 +495,7 @@ void RoomManager::setConnection(NeoChatConnection *connection)
     m_connection = connection;
 
     if (m_connection != nullptr) {
+        m_lastRoomConfig = m_config->group(m_connection->userId()).group(u"LastOpenRoom"_s);
         connect(m_connection, &NeoChatConnection::showMessage, this, &RoomManager::showMessage);
         connect(m_connection, &NeoChatConnection::createdRoom, this, [this](Quotient::Room *room) {
             resolveResource(room->id());
@@ -518,15 +519,18 @@ void RoomManager::setCurrentSpace(const QString &spaceId, bool setRoom)
 
     // This need to happen before the signal so TreeView.expandRecursively() can work nicely.
     m_sortFilterRoomTreeModel->setActiveSpaceId(m_currentSpaceId);
-    m_sortFilterRoomTreeModel->setMode(m_currentSpaceId == u"DM"_s ? SortFilterRoomTreeModel::DirectChats : SortFilterRoomTreeModel::Rooms);
+
+    if (m_currentSpaceId == u"DM") {
+        m_sortFilterRoomTreeModel->setMode(SortFilterRoomTreeModel::DirectChats);
+    } else if (m_currentSpaceId.isEmpty()) {
+        m_sortFilterRoomTreeModel->setMode(SortFilterRoomTreeModel::Rooms);
+    } else {
+        m_sortFilterRoomTreeModel->setMode(SortFilterRoomTreeModel::All);
+    }
 
     Q_EMIT currentSpaceChanged();
     if (m_connection) {
-        if (spaceId.isEmpty()) {
-            m_lastSpaceConfig.writeEntry(m_connection->userId(), u"Home"_s);
-        } else {
-            m_lastSpaceConfig.writeEntry(m_connection->userId(), spaceId);
-        }
+        m_lastRoomConfig.writeEntry(u"lastSpace"_s, spaceId.isEmpty() ? u"Home"_s : spaceId);
     }
 
     if (!setRoom) {
@@ -534,22 +538,20 @@ void RoomManager::setCurrentSpace(const QString &spaceId, bool setRoom)
     }
 
     // We intentionally don't want to open the last room on mobile
-    if (!m_isMobile) {
-        QString configSpaceId = spaceId;
-        // We can't have empty keys in KConfig, so it's stored as "Home"
-        if (spaceId.isEmpty()) {
-            configSpaceId = u"Home"_s;
-        }
-
-        const auto &lastRoom = m_lastRoomConfig.readEntry(configSpaceId, QString());
-        if (lastRoom.isEmpty()) {
-            if (spaceId != u"DM"_s && spaceId != u"Home"_s) {
-                resolveResource(spaceId, "no_join"_L1);
-            }
-        } else {
-            resolveResource(lastRoom, "no_join"_L1);
-        }
+    if (m_isMobile) {
+        return;
     }
+
+    // We can't have empty keys in KConfig, so it's stored as "Home"
+    if (const auto &lastRoom = m_lastRoomConfig.readEntry(spaceId.isEmpty() ? u"Home"_s : spaceId, QString()); !lastRoom.isEmpty()) {
+        resolveResource(lastRoom, "no_join"_L1);
+        return;
+    }
+    if (!spaceId.isEmpty() && spaceId != u"DM"_s) {
+        resolveResource(spaceId, "no_join"_L1);
+        return;
+    }
+    setCurrentRoom({});
 }
 
 void RoomManager::setCurrentRoom(const QString &roomId)
@@ -588,7 +590,8 @@ void RoomManager::setCurrentRoom(const QString &roomId)
         return;
     }
     if (m_currentRoom->isDirectChat()) {
-        if (m_currentSpaceId != "DM"_L1) {
+        const auto roomsInSpace = SpaceHierarchyCache::instance().getRoomListForSpace(m_currentSpaceId, false);
+        if (!roomsInSpace.contains(m_currentRoom->id()) && m_currentSpaceId != "DM"_L1) {
             setCurrentSpace("DM"_L1, false);
         }
         return;
