@@ -93,6 +93,9 @@ QVariant UserListModel::data(const QModelIndex &index, int role) const
         return QVariant::fromValue(memberId);
     }
     if (role == PowerLevelRole) {
+        if (m_currentRoom->isCreator(memberId)) {
+            return std::numeric_limits<int>::max();
+        }
         auto plEvent = m_currentRoom->currentState().get<RoomPowerLevelsEvent>();
         if (!plEvent) {
             return 0;
@@ -100,6 +103,10 @@ QVariant UserListModel::data(const QModelIndex &index, int role) const
         return plEvent->powerLevelForUser(memberId);
     }
     if (role == PowerLevelStringRole) {
+        if (m_currentRoom->roomCreatorHasUltimatePowerLevel() && m_currentRoom->isCreator(memberId)) {
+            return i18nc("@info the person that created this room", "Creator");
+        }
+
         auto pl = m_currentRoom->currentState().get<RoomPowerLevelsEvent>();
         // User might not in the room yet, in this case pl can be nullptr.
         // e.g. When invited but user not accepted or denied the invitation.
@@ -114,6 +121,9 @@ QVariant UserListModel::data(const QModelIndex &index, int role) const
                      PowerLevel::nameForLevel(PowerLevel::levelForValue(userPl)),
                      userPl);
     }
+    if (role == IsCreatorRole) {
+        return m_currentRoom->isCreator(memberId);
+    }
 
     return {};
 }
@@ -217,6 +227,7 @@ QHash<int, QByteArray> UserListModel::roleNames() const
     roles[ObjectRole] = "user";
     roles[PowerLevelRole] = "powerLevel";
     roles[PowerLevelStringRole] = "powerLevelString";
+    roles[IsCreatorRole] = "isCreator";
 
     return roles;
 }
