@@ -8,9 +8,7 @@
 #include <Quotient/events/encryptedevent.h>
 #include <Quotient/events/roommessageevent.h>
 #include <Quotient/events/stickerevent.h>
-#if Quotient_VERSION_MINOR > 9 || (Quotient_VERSION_MINOR == 9 && Quotient_VERSION_PATCH > 1)
 #include <Quotient/thread.h>
-#endif
 
 #include <KFormat>
 
@@ -20,6 +18,7 @@
 #include "eventhandler.h"
 #include "events/pollevent.h"
 #include "models/reactionmodel.h"
+#include "models/eventmessagecontentmodel.h"
 #include "neochatroommember.h"
 
 using namespace Quotient;
@@ -32,7 +31,7 @@ bool MessageModel::m_threadsEnabled = false;
 MessageModel::MessageModel(QObject *parent)
     : QAbstractListModel(parent)
 {
-    connect(this, &MessageModel::newEventAdded, this, &MessageModel::createEventObjects);
+    connect(this, &MessageModel::newEventAdded, this, &MessageModel::createEventObjects, Qt::QueuedConnection);
 
     connect(this, &MessageModel::modelAboutToReset, this, [this]() {
         m_resetting = true;
@@ -42,7 +41,7 @@ MessageModel::MessageModel(QObject *parent)
     });
 
     connect(this, &MessageModel::threadsEnabledChanged, this, [this]() {
-        Q_EMIT dataChanged(index(0), index(rowCount() - 1), {IsThreadedRole});
+        Q_EMIT dataChanged(index(0), index(rowCount() - 1), {DelegateTypeRole, ContentModelRole, IsThreadedRole, SpecialMarksRole});
     });
 }
 
@@ -142,14 +141,15 @@ QVariant MessageModel::data(const QModelIndex &idx, int role) const
 
     if (role == ContentModelRole) {
         if (event->get().is<EncryptedEvent>() || event->get().is<PollStartEvent>() || event->get().is<StickerEvent>()) {
-            return QVariant::fromValue<MessageContentModel *>(ContentProvider::self().contentModelForEvent(m_room, event->get().id()));
+            return QVariant::fromValue<EventMessageContentModel *>(ContentProvider::self().contentModelForEvent(m_room, event->get().id()));
         }
 
         auto roomMessageEvent = eventCast<const RoomMessageEvent>(&event.value().get());
         if (m_threadsEnabled && roomMessageEvent && roomMessageEvent->isThreaded()) {
-            return QVariant::fromValue<MessageContentModel *>(ContentProvider::self().contentModelForEvent(m_room, roomMessageEvent->threadRootEventId()));
+            return QVariant::fromValue<EventMessageContentModel *>(
+                ContentProvider::self().contentModelForEvent(m_room, roomMessageEvent->threadRootEventId()));
         }
-        return QVariant::fromValue<MessageContentModel *>(ContentProvider::self().contentModelForEvent(m_room, &event->get()));
+        return QVariant::fromValue<EventMessageContentModel *>(ContentProvider::self().contentModelForEvent(m_room, &event->get()));
     }
 
     if (role == GenericDisplayRole) {
@@ -194,19 +194,12 @@ QVariant MessageModel::data(const QModelIndex &idx, int role) const
         }
 
         auto roomMessageEvent = eventCast<const RoomMessageEvent>(&event.value().get());
-#if Quotient_VERSION_MINOR > 9 || (Quotient_VERSION_MINOR == 9 && Quotient_VERSION_PATCH > 1)
         if (m_threadsEnabled && roomMessageEvent && (roomMessageEvent->isThreaded() || m_room->threads().contains(event.value().get().id()))) {
             const auto &thread = m_room->threads().value(roomMessageEvent->isThreaded() ? roomMessageEvent->threadRootEventId() : event.value().get().id());
             if (thread.latestEventId != event.value().get().id()) {
                 return EventStatus::Hidden;
             }
         }
-#else
-        if (roomMessageEvent && roomMessageEvent->isThreaded() && roomMessageEvent->threadRootEventId() != event.value().get().id() && m_threadsEnabled) {
-            return EventStatus::Hidden;
-        }
-#endif
-
         return EventStatus::Normal;
     }
 
@@ -433,15 +426,9 @@ void MessageModel::createEventObjects(const Quotient::RoomEvent *event)
     }
 
     auto eventId = event->id();
-    auto senderId = event->senderId();
     if (eventId.isEmpty()) {
         eventId = event->transactionId();
     }
-    // A pending event might not have a sender ID set yet but in that case it must
-    // be the local member.
-    if (senderId.isEmpty()) {
-        senderId = m_room->localMember().id();
-    }
 
     // ReadMarkerModel handles updates to add and remove markers, we only need to
     // handle adding and removing whole models here.
